<?php
/**
* Plugin Name: Convert WPBakery to Elementor
* Plugin URI: https://www.wpbakery-to-elementor.com/
* Author: Morten Rikard
* Description: Convert and migrate pages from WPBakery to Elementor
* Version: 1.0
* License: GPL2
* License URI: http://www.gnu.org/licenses/gpl-2.0.txt
* Text Domain: convert-wpbakery-to-elementor
**/

if (! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

defined ( 'ABSPATH' ) or die( 'Unauthorized Access' );


if (!class_exists('wpbakery_to_elementor')):
    class wpbakery_to_elementor
    { 
        /*
        * AutoLoad Hooks
        */
        public function __construct() 
        {
        	$opt = get_option('wpbakery_to_elementor_options');
            register_activation_hook(__FILE__, array(&$this, 'wpbakery_to_elementor_install'));
            register_deactivation_hook(__FILE__, array(&$this, 'wpbakery_to_elementor_uninstall'));
            add_action('admin_menu', array(&$this, 'wpbakery_to_elementor_options_page'));
            add_filter('plugin_action_links', array(&$this, 'wpbakery_to_elementor_plugin_action_links'), 10, 2);
            add_action('admin_action_dt_wpbakery_to_elementor_as_draft', array(&$this, 'dt_wpbakery_to_elementor_as_draft'));
            add_filter('post_row_actions', array(&$this, 'dt_wpbakery_to_elementor_link'), 10, 2);
            add_filter('page_row_actions', array(&$this, 'dt_wpbakery_to_elementor_link'), 10, 2);
            add_action('pre_get_posts', array($this, 'wpbakery_to_elementor_exclude_placeholder_from_medialibrary'));
        }


        /*
        * Activation Hook
        */
        public function wpbakery_to_elementor_install()
        {
            $defaultsettings = array(
                'wpbakery_to_elementor_status' => 'draft',
                'wpbakery_to_elementor_redirect' => 'to_list',
                'wpbakery_to_elementor_suffix' => '',
                'wpbakery_to_elementor_licensekey' => '',
                'wpbakery_to_elementor_editor' => 'classic',
            );
            $opt = get_option('wpbakery_to_elementor_options');
            if (!isset($opt['wpbakery_to_elementor_status'])) {
                update_option('wpbakery_to_elementor_options', $defaultsettings);
            }

            // Register the image in the media library and get the attachment ID
            $attachment_id =  $this->wpbakery_to_elementor_imageplaceholder();
            // Store the attachment ID in the options table for future reference
            if ($attachment_id) {
                update_option('wpbakery_to_elementor_placeholder_image_id', $attachment_id);
            }
        }

        /*
        * DeActivation Hook
        */
        public function wpbakery_to_elementor_uninstall()
        {
             // Retrieve the attachment ID stored in the options table
            $attachment_id = get_option('wpbakery_to_elementor_placeholder_image_id');
            // Check if the attachment exists, then delete it
            if ($attachment_id) {
                wp_delete_attachment($attachment_id, true); // true means force delete, including files
                delete_option('wpbakery_to_elementor_placeholder_image_id');  // Clean up the option
            }
        }


        /*
        Action Links
        */
        public function wpbakery_to_elementor_plugin_action_links($links, $file)
        {
            if ($file == plugin_basename(__FILE__)) {
                $wpbakery_to_elementor_links = '<a href="'.esc_url(get_admin_url()).'tools.php?page=wpbakery-to-eIementor">'.__('Settings', 'convert-wpbakery-to-elementor').'</a>';
                array_unshift($links, $wpbakery_to_elementor_links);
            }
            return $links;
        }

        /*
        * Admin Menu
        */
        public function wpbakery_to_elementor_options_page()
        {
            add_submenu_page(
                'tools.php',  // Parent slug for Tools menu
                'WPBakery to Elementor', // Page title
                'Convert WPBakery to Elementor',  // Menu title
                'manage_options', // Capability
                'wpbakery-to-eIementor', // URL
                 array(&$this, 'wpbakery_to_elementor_admin_settings') // Function that will render the page content
            );
        }

        /*
        * WPBakery to Elementor Admin Settings
        */
        public function wpbakery_to_elementor_admin_settings()
        {
            if (current_user_can('manage_options')) {
                include 'inc/admin-settings.php';
            }
        }


        /*
        * Main function
        */
        public function dt_wpbakery_to_elementor_as_draft()
        {
           // get Nonce value
           if(isset($_REQUEST['nonce'])) {
           $nonce = sanitize_text_field(wp_unslash($_REQUEST['nonce']));
           
                // get the original post id
               if(isset($_GET['post'])) {
                   $post_id = intval($_GET['post']);
                   $post = get_post($post_id);
                   $current_user_id = get_current_user_id();
                    if(wp_verify_nonce( $nonce, 'dt-wpbakery-to-elementor-'.$post_id)) {
                            if (current_user_can('manage_options') || current_user_can('edit_posts') || current_user_can('edit_others_posts')) {
                                // primary
                                $this->wpbakery_to_elementor_prepare_ajax($post_id);
                            }
                            else {
                                wp_die(esc_html__('Unauthorized Access.', 'convert-wpbakery-to-elementor'));

                            }
                      }
                    else {
                        wp_die(esc_html__('Security check issue, Please try again.','convert-wpbakery-to-elementor'));
                    } 
                }
            }

        }


        public function wpbakery_to_elementor_prepare_ajax($post_id) {
            $post = get_post($post_id);
            $opt = get_option('wpbakery_to_elementor_options');


            if (isset($post) && $post != null) {
                if (isset($opt['wpbakery_to_elementor_editor']) && $opt['wpbakery_to_elementor_editor'] == 'gutenberg') {
                    $wpbakery_to_elementor_content = wp_slash($post->post_content);
                } else {
                    $wpbakery_to_elementor_content = $post->post_content;
                }
                $wpbakery_to_elementor_content_encoded = $this->wpbakery_to_elementor_replacelinebreaks($wpbakery_to_elementor_content);
                $wpbakery_to_elementor_content_images = $this->wpbakery_to_elementor_getimages($wpbakery_to_elementor_content_encoded);
                $this->wpbakery_to_elementor_ajax_request($post_id,$wpbakery_to_elementor_content_images);
            }

        }

                
        // Handle the AJAX request
        function wpbakery_to_elementor_ajax_request($post_id,$wpb_shortcode) {
            $opt = get_option('wpbakery_to_elementor_options');
            $wpbakery_to_elementor_referrer = "";
            $wpbakery_to_elementor_licensekey = "";
            if (isset($_SERVER['HTTP_HOST'])) {
                $wpbakery_to_elementor_referrer = sanitize_text_field(wp_unslash($_SERVER['HTTP_HOST']));
            }
            if (isset($opt['wpbakery_to_elementor_licensekey'])) {
                $wpbakery_to_elementor_licensekey = esc_attr($opt['wpbakery_to_elementor_licensekey']);
            }

            $wpbe_data_to_send = array(
                'shortcode' => $wpb_shortcode,
                'images' => 1,
                'license' => $wpbakery_to_elementor_licensekey
            );
            $response = wp_remote_post('https://convert.wpbakery-to-elementor.com', array(
                'method'    => 'POST',
                'body'      => $wpbe_data_to_send,
                'headers'   => array(
                    'Content-Type' => 'application/x-www-form-urlencoded',
                    'Referer'      => $wpbakery_to_elementor_referrer
                    )
            ));

            if (is_wp_error($response)) {
                wp_send_json_error(array('message' => 'Failed to retrieve content.'));
            } else {
                $post_response = wp_remote_retrieve_body($response);
                    
                    // Get response body and decode
                    $post_response = json_decode($post_response, true);

                    // Check if the response is a valid JSON and not null
                    if (json_last_error() === JSON_ERROR_NONE) {
                        // Process the response data
                        if ((isset($post_response['success']) && $post_response['success'] === true) || $post_response['status'] == 'success') {
                            // Extract the 'data' array
                            $post_response_message = $post_response['message'];
                            $post_response_data = $post_response['data'];
                        } else {
                          if (isset($post_response['error'])) {
                            wp_die(esc_html($post_response['error']['message']));
                          }
                        }
                    }

                $this->wpbakery_to_elementor_convert_post($post_id,$post_status_update='',$post_response_data);
            }
        }
                

        /**
         * Create post
         */
        public function wpbakery_to_elementor_convert_post($post_id,$post_status_update='',$post_elementor_content)
        {
            global $wpdb;
            $opt = get_option('wpbakery_to_elementor_options');
            $suffix = isset($opt['wpbakery_to_elementor_suffix']) && !empty($opt['wpbakery_to_elementor_suffix']) ? ' -- '.esc_attr($opt['wpbakery_to_elementor_suffix']) : '';
                if($post_status_update == ''){
                    $post_status = !empty($opt['wpbakery_to_elementor_status']) ? esc_attr($opt['wpbakery_to_elementor_status']) : 'draft';
                }else{
                    $post_status =  $post_status_update;
                }
            $redirectit = !empty($opt['wpbakery_to_elementor_redirect']) ? esc_attr($opt['wpbakery_to_elementor_redirect']) : 'to_list';
            $returnpage = '';            
            $post = get_post($post_id);
            /*
            * if you don't want current user to be the new post author,
            * then change next couple of lines to this: $new_post_author = $post->post_author;
            */
            $current_user = wp_get_current_user();
            $new_post_author = $current_user->ID;


            /*
            * if post data exists, create the post duplicate
            */
            if (isset($post) && $post != null) {
                /*
                * new post data array
                */
                $args = array(
                     'comment_status' => $post->comment_status,
                     'ping_status' => $post->ping_status,
                     'post_author' => $new_post_author,
                     'post_content' => (isset($opt['wpbakery_to_elementor_editor']) && $opt['wpbakery_to_elementor_editor'] == 'gutenberg') ? wp_slash($post->post_content) : $post->post_content,
                     'post_excerpt' => $post->post_excerpt,
                     'post_parent' => $post->post_parent,
                     'post_password' => $post->post_password,
                     'post_status' => $post_status,
                     'post_title' => $post->post_title.$suffix,
                     'post_type' => $post->post_type,
                     'to_ping' => $post->to_ping,
                     'menu_order' => $post->menu_order,
                 );
                /*
                * insert the post by wp_insert_post() function
                */
                $new_post_id = wp_insert_post($args);
                if(is_wp_error($new_post_id)){
                    wp_die(esc_html($new_post_id->get_error_message()));
                }
               
                /*
                * get all current post terms ad set them to the new post draft
                */
                $taxonomies = array_map('sanitize_text_field',get_object_taxonomies($post->post_type));
                if (!empty($taxonomies) && is_array($taxonomies)) {
                 foreach ($taxonomies as $taxonomy) {
                     $post_terms = wp_get_object_terms($post_id, $taxonomy, array('fields' => 'slugs'));
                     wp_set_object_terms($new_post_id, $post_terms, $taxonomy, false);
                 }
                }

                /*
                * duplicate all post meta
                */
                $isElementorPresent = false;
                $post_meta_keys = get_post_custom_keys( $post_id );
                if(!empty($post_meta_keys)){
                    foreach ( $post_meta_keys as $meta_key ) {
                        $meta_values = get_post_custom_values( $meta_key, $post_id );
                        foreach ( $meta_values as $meta_value ) {
                            //echo "meta_key:" . $meta_key . " > " . $meta_value . "\n<br />";
                            if($meta_key == "ast-site-content-layout" && $meta_value == "default") {
                                // Astra, default layout
                                $meta_value = "full-width-container";
                            }
                            $meta_value = maybe_unserialize( $meta_value );
                            update_post_meta( $new_post_id, $meta_key, wp_slash( $meta_value ) );
                            
                            if (strpos($meta_key, '_elementor') !== false) {
                                if (!empty($meta_value)) {
                                    $isElementorPresent = true;
                                }
                                
                            }
                        }
                    }
                }

                if (!$isElementorPresent) {
                    //add_post_meta( $new_post_id, "ast-site-content-layout", wp_slash( "full-width-container" ) );
                    add_post_meta( $new_post_id, "site-content-layout", wp_slash( "page-builder" ) );
                    //add_post_meta( $new_post_id, "_elementor_page_assets ", wp_slash( "a:0:{}" ) );
                    add_post_meta( $new_post_id, "_elementor_edit_mode", wp_slash( "builder" ) );
                    add_post_meta( $new_post_id, "_elementor_template_type", wp_slash( "wp-page" ) );
                    if ( defined( 'ELEMENTOR_VERSION' ) ) {
                        $elementor_version = ELEMENTOR_VERSION;
                        add_post_meta( $new_post_id, "_elementor_version", wp_slash( $elementor_version ) );
                    } else {
                        add_post_meta( $new_post_id, "_elementor_version", wp_slash( "3.21.8" ) );
                    }
                    add_post_meta( $new_post_id, "_elementor_data", wp_slash( $post_elementor_content ) );
                    //echo "_elementor_data: $post_elementor_content \n<br />";

                }

                $post_meta_keys = get_post_custom_keys( $new_post_id );
                if(!empty($post_meta_keys)){
                    foreach ( $post_meta_keys as $meta_key ) {
                        $meta_values = get_post_custom_values( $meta_key, $new_post_id );
                        foreach ( $meta_values as $meta_value ) {
                            $meta_value = maybe_unserialize( $meta_value );
                           }
                    }
                }

                /**
                 * Elementor fixes
                 */
                 if(is_plugin_active( 'elementor/elementor.php' )){
                        $css = Elementor\Core\Files\CSS\Post::create( $new_post_id );
                        $css->update();
                 }

                /*
                * Redirecting
                */
                if ($post->post_type != 'post'){
                    $returnpage = '?post_type='.$post->post_type;
                }
                if (!empty($redirectit) && $redirectit == 'to_list'){
                    wp_redirect(esc_url_raw(admin_url('edit.php'.$returnpage)));
                } elseif (!empty($redirectit) && $redirectit == 'to_page'){
                    wp_redirect(esc_url_raw(admin_url('post.php?post='.$new_post_id."&action=elementor")));
                } else {
                    wp_redirect(esc_url_raw(admin_url('edit.php'.$returnpage)));
                }

             exit;

            } 
            else {
                $post_id = absint( $post_id );
                /* translators: %d: Post ID */
                wp_die(esc_html(sprintf(__('Error! Post creation failed, could not find original post: %d', 'convert-wpbakery-to-elementor' ), $post_id)));
            }
        }


        function wpbakery_to_elementor_replacelinebreaks($string) {
            // Define placeholders for each line break type
            $placeholders = [
                "\r\n" => '<CRLF>',
                "\n" => '<LF>',
                "\u{0085}" => '<NEL>',
                "\u{2028}" => '<LS>',
                "\u{2029}" => '<PS>',
            ];
            // Replace each type of line break with its placeholder
            $encodedString = $string;
            foreach ($placeholders as $lineBreak => $placeholder) {
                $encodedString = str_replace($lineBreak, $placeholder, $encodedString);
            }
            return $encodedString;
        }


        function wpbakery_to_elementor_imageplaceholder() {
            global $wp_filesystem;
            // Initialize WP_Filesystem
            if ( ! function_exists( 'WP_Filesystem' ) ) {
                require_once ABSPATH . 'wp-admin/includes/file.php';
            }
            WP_Filesystem(); // Initialize the WP_Filesystem object
            // Path to the image in the plugin's assets folder
            $image_url = plugins_url('assets/cwte-image-placeholder.jpg', __FILE__);
            // Use wp_remote_get() to fetch the image data
            $response = wp_remote_get($image_url);
            // Check if the request was successful
            if (is_wp_error($response)) {
                return false; // Handle error accordingly
            }
            // Get the image body content
            $image_data = wp_remote_retrieve_body($response);
            $upload_dir = wp_upload_dir();
            $filename = basename($image_url);
            // Prepare the upload path in the uploads directory
            $upload_file = trailingslashit($upload_dir['path']) . $filename;
            if ( $wp_filesystem->put_contents( $upload_file, $image_data, FS_CHMOD_FILE ) ) {
                // File successfully written
                $wp_filetype = wp_check_filetype($filename, null);
                // Prepare the attachment details
                $attachment = array(
                    'post_mime_type' => $wp_filetype['type'],
                    'post_title'     => '',
                    'post_content'   => '',
                    'post_status'    => 'private'  // Set status to private to hide from the media library, previous: inherit
                );

                // Insert the attachment into the WordPress database
                $attach_id = wp_insert_attachment($attachment, $upload_file);
                // Include the necessary WordPress functions to handle metadata
                require_once(ABSPATH . 'wp-admin/includes/image.php');
                // Generate metadata for the attachment and update the database record
                $attach_data = wp_generate_attachment_metadata($attach_id, $upload_file);
                wp_update_attachment_metadata($attach_id, $attach_data);
                add_post_meta($attach_id, '_cwte_attachment', '1');
                return $attach_id; // Return the attachment ID
            }
            return false; // Return false if the file couldn't be uploaded
        }

        function wpbakery_to_elementor_exclude_placeholder_from_medialibrary( $query ) {
            // Only modify the query in the admin area and for the media library
            if (is_admin() && isset($query->query['post_type']) && $query->query['post_type'] === 'attachment') {
                // Exclude attachments with the '_cwte_hidden_attachment' meta field
                $meta_query = array(
                    array(
                        'key'     => '_cwte_attachment',
                        'compare' => 'NOT EXISTS',
                    ),
                );
                $query->set('meta_query', $meta_query);
            }
        }
        

        function wpbakery_to_elementor_getimages($string) {
            // Pattern to match vc_single_image and its attributes
            $single_image_pattern = '/\[vc_single_image([^\]]*)\]/s';
            // Pattern to match the image attribute within vc_single_image
            $image_attribute_pattern = '/image="([^"]*)"/';
           
            // Replace function
            $string = preg_replace_callback($single_image_pattern, function($matches) use ($image_attribute_pattern) {
                $attributes_str = $matches[1];
                $attachment_id = get_option('wpbakery_to_elementor_placeholder_image_id');
                if ($attachment_id) {
                    if (preg_match($image_attribute_pattern, $attributes_str, $attribute_match)) {
                        $attributes_str = preg_replace($image_attribute_pattern, 'image="' . $attachment_id . '"', $attributes_str);
                    }
                    $attachment_url = wp_get_attachment_url($attachment_id);
                    if ($attachment_url) {
                        $new_attributes_str = $attributes_str . ' src="' . $attachment_url . '"';
                        return '[vc_single_image' . $new_attributes_str . ']';
                    }
                 }
                 // If not found, return the original shortcode
                 return $matches[0];
            }, $string);
            return $string;
        }

        /*
         * Add the convert link to action list for post_row_actions
         */
        public function dt_wpbakery_to_elementor_link($actions, $post)
        {
            // Skip acf-field-group post type
            if($post->post_type == 'acf-field-group'){
                return $actions;
            }
            $opt = get_option('wpbakery_to_elementor_options');
            $post_status = !empty($opt['wpbakery_to_elementor_status']) ? esc_attr($opt['wpbakery_to_elementor_status']) : 'draft';
            
            if (current_user_can('edit_posts')) {
                // Only show link to WPBakery pages
                if ( $post && preg_match( '/vc_row/', $post->post_content ) ) {
                  $actions['wpbakerycopy'] = isset($post) ? '<a href="admin.php?action=dt_wpbakery_to_elementor_as_draft&amp;post='.intval($post->ID).'&amp;nonce='.wp_create_nonce( 'dt-wpbakery-to-elementor-'.intval($post->ID) ).'" title="'.__('Create a Elementor ', 'convert-wpbakery-to-elementor').$post_status.'" rel="permalink">'.__('Create Elementor copy', 'convert-wpbakery-to-elementor').'</a>' : '';
                }
            }
            return $actions;
        }


        /*
         * Redirect function
        */
        public static function wte_redirect($url)
        {        
            $web_url = esc_url($url);
            wp_register_script( 'wte-setting-redirect', '');
            wp_enqueue_script( 'wte-setting-redirect' );
            wp_add_inline_script(
            'wte-setting-redirect',
            ' window.location.href="'.$web_url.'" ;'
        );
        }


        /*
        Custom Assets
        */
        public function custom_assets() {
            wp_enqueue_style( 'convert-wpbakery-to-elementor', plugins_url( '/css/wpbakery_to_elementor.css', __FILE__ ) );
            wp_enqueue_script( 'convert-wpbakery-to-elementor', plugins_url( '/js/wpbakery_to_elementor.js', __FILE__ ) );
        }

    } //  end class wpbakery_to_elementor

new wpbakery_to_elementor();
endif;